module Echo
  module Configuration
    # An array of valid keys in the options hash when configuring a {Echo::API}
    VALID_OPTIONS_KEYS = [
      :endpoint,
      :format,
      :debug,
      :user,
      :password,
      :backplane_user,
      :backplane_password,
      :search_endpoint].freeze 

    # An array of valid request/response formats
    #
    # @note Not all methods support the XML format.
    VALID_FORMATS = [
      :json,
      :xml].freeze

    # By default, don't set an application key
    DEFAULT_CONSUMER_KEY = nil

    # By default, don't set an application secret
    DEFAULT_CONSUMER_SECRET = nil

    # The endpoint that will be used to connect if none is set
    #
    # @note This is configurable in case you want to use HTTP instead of HTTPS, specify a different API version, or use a Echo-compatible endpoint.
    DEFAULT_ENDPOINT = 'http://api.echoenabled.com/v1'.freeze
    
    # The response format appended to the path and sent in the 'Accept' header if none is set
    #
    # @note JSON is preferred over XML because it is more concise and faster to parse.
    DEFAULT_FORMAT = :json

    # By default, don't set a user oauth token
    DEFAULT_OAUTH_TOKEN = nil

    # By default, don't set a user oauth secret
    DEFAULT_OAUTH_TOKEN_SECRET = nil

    # By default, don't use a proxy server
    DEFAULT_PROXY = nil

    # The search endpoint that will be used to connect if none is set
    #
    # @note This is configurable in case you want to use HTTP instead of HTTPS or use a Echo-compatible endpoint.
    # @see http://status.net/wiki/Echo-compatible_API
    DEFAULT_SEARCH_ENDPOINT = 'https://search.echo.com/'.freeze

    # The default user for basic authoriation
    DEFAULT_USER = nil

    # The default password for basic authorization
    DEFAULT_PASSWORD = nil

    # @private
    attr_accessor *VALID_OPTIONS_KEYS

    # When this module is extended, set all configuration options to their default values
    def self.extended(base)
      base.reset
    end

    # Convenience method to allow configuration options to be set in a block
    def configure
      yield self
    end

    # Create a hash of options and their values
    def options
      options = {}
      VALID_OPTIONS_KEYS.each{|k| options[k] = send(k) }
      options
    end

    # Reset all configuration options to defaults
    def reset
      self.debug              = false
      self.endpoint           = DEFAULT_ENDPOINT
      self.format             = DEFAULT_FORMAT
      self.search_endpoint    = DEFAULT_SEARCH_ENDPOINT
      self.user               = DEFAULT_USER
      self.password           = DEFAULT_PASSWORD
      self
    end
  end
end
